/**
 * <copyright>
 *
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 *   IBM - Initial API and implementation
 *
 * </copyright>
 *
 * $Id$
 */
package org.eclipse.jet.compiler;


import java.util.Map;

import org.eclipse.jet.taglib.TagDefinition;


/**
 * The root AST object
 *
 */
class JET2AST
{

  /**
   * Create an instance
   */
  public JET2AST()
  {
    // do nothing
  }

  /**
   * Create a text element in the AST
   * @param chars the text
   * @return the new element
   */
  TextElement createTextElement(char[] chars)
  {
    return new TextElement(this, chars);
  }

  /**
   * Create a new Directive element in the AST
   * @param line the start line
   * @param colOffset the offset within the line of the element's start.
   * @param start the start offset (doc relative)
   * @param end the end offset (doc relative)
   * @param directiveName the directive name
   * @param attributes a map off attribute names and values
   * @return the new element
   */
  JET2Directive createDirective(int line, int colOffset, int start, int end, String directiveName, Map attributes)
  {
    return new JET2Directive(this, line, colOffset, start, end, directiveName, attributes);
  }

  /**
   * Create a new Java Expression element in the AST
   * @param line the start line
   * @param colOffset the offset within the line of the element's start.
   * @param start the offset of the expression element (doc relative)
   * @param end the end offset of the expression element (doc relative)
   * @param javaStart the offset of the Java code (doc relative)
   * @param javaEnd the end offset of the Java code
   * @param javaContent the Java content
   * @return the new element
   */
  JET2Expression createExpression(int line, int colOffset, int start, int end, int javaStart, int javaEnd, char[] javaContent)
  {
    return new JET2Expression(this, line, colOffset, start, end, javaStart, javaEnd, javaContent);
  }

  /**
   * Create a new Java Scriptlet element in the AST
   * @param line the start line
   * @param colOffset the offset within the line of the element's start.
   * @param start the offset of the scriptlet element (doc relative)
   * @param end the end offset of the scriplet element (doc relative)
   * @param javaStart the offset of the Java code (doc relative)
   * @param javaEnd the end offset of the Java code
   * @param javaContent the Java content
   * @return the new element
   */
  JET2Scriptlet createScriplet(int line, int colOffset, int start, int end, int javaStart, int javaEnd, char[] javaContent)
  {
    return new JET2Scriptlet(this, line, colOffset, start, end, javaStart, javaEnd, javaContent);
  }

  /**
   * Create a new Java Declaration element in the AST
   * @param line the start line
   * @param colOffset the offset within the line of the element's start.
   * @param start the offset of the declaration element (doc relative)
   * @param end the end offset of the declaration element (doc relative)
   * @param javaStart the offset of the Java code (doc relative)
   * @param javaEnd the end offset of the Java code
   * @param javaContent the Java content
   * @return the new element
   */
  JET2Declaration createDeclaration(int line, int colOffset, int start, int end, int javaStart, int javaEnd, char[] javaContent)
  {
    return new JET2Declaration(this, line, colOffset, start, end, javaStart, javaEnd, javaContent);
  }

  /**
   * Create a new empty XML element in the AST
   * @param line the start line of the element
   * @param colOffset the offset within the line of the element's start.
   * @param start the start offset of the element (doc relative)
   * @param end the end offset of the element (doc relative)
   * @param tagName the QName of the element
   * @param attributeMap a Map off element attribute names and values
   * @param td the TagDefinition of the element
   * @return the new element
   */
  XMLEmptyElement createXMLEmptyElement(int line, int colOffset, int start, int end, String tagName, Map attributeMap, TagDefinition td)
  {
    return new XMLEmptyElement(this, line, colOffset, start, end, tagName, attributeMap, td);
  }

  /**
   * Create a new XML element with body in the AST
   * @param line the start line of the element
   * @param colOffset the offset within the line of the element's start.
   * @param start the start offset of the element (doc relative)
   * @param end the end offset of the element (doc relative)
   * @param tagName the QName of the element
   * @param attributeMap a Map off element attribute names and values
   * @param td the TagDefinition of the tag
   * @return the new element
   */
  XMLBodyElement createXMLBodyElement(int line, int colOffset, int start, int end, String tagName, Map attributeMap, TagDefinition td)
  {
    return new XMLBodyElement(this, line, colOffset, start, end, tagName, attributeMap, td);
  }

  /**
   * Create a new Comment element in the AST
   * @param line the start line of the element
   * @param colOffset the offset within the line of the element's start.
   * @param start the start offset of the element (doc relative)
   * @param end the end offset of the element (doc relative)
   * @param commentStart the start offset of the comment text (doc relative)
   * @param commentEnd the end offset of the comment text (doc relative)
   * @param comment the comment text
   * @return the new element
   */
  Comment createComment(int line, int colOffset, int start, int end, int commentStart, int commentEnd, char[] comment)
  {
    return new Comment(this, line, colOffset, start, end, commentStart, commentEnd, comment);
  }

  /**
   * Create a new XML end element corresponding to the end of a XMLBodyElement.
   * @param line
   * @param col
   * @param start
   * @param end
   * @param tagName
   * @param startTag
   * @return
   */
  XMLBodyElementEnd createXMLBodyElementEnd(int line, int col, int start, int end, String tagName)
  {
    return new XMLBodyElementEnd(this, line, col, start, end);
  }

}
