/**
 * <copyright>
 *
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 *   IBM - Initial API and implementation
 *
 * </copyright>
 *
 * $Id$
 */

package org.eclipse.jet.internal.taglib;


import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.jet.internal.InternalJET2Platform;
import org.eclipse.jet.internal.l10n.JET2Messages;
import org.eclipse.jet.taglib.CustomTag;
import org.eclipse.jet.taglib.CustomTagKind;
import org.eclipse.jet.taglib.TagAttributeDefinition;
import org.eclipse.jet.taglib.TagDefinition;
import org.eclipse.jet.taglib.TagLibrary;


/**
 * Implementation of TagDefinition.
 *
 */
public final class TagDefinitionImpl implements TagDefinition
{

  private static final String CLASS__ATTR = "class"; //$NON-NLS-1$

  private static final String USE__ATTR = "use"; //$NON-NLS-1$

  private static final String USE__VALUE__REQUIRED = "required"; //$NON-NLS-1$

  private static final String NAME__ATTR = "name"; //$NON-NLS-1$

  private static final String ATTRIBUTE__ELEMENT = "attribute"; //$NON-NLS-1$

  private static final String ALLOW_AS_EMPTY__ATTR = "allowAsEmpty"; //$NON-NLS-1$

  private static final String CONTENT_PROCESSING__VALUE__CUSTOM = "custom"; //$NON-NLS-1$

  private static final String CONTENT_PROCESSING__ATTR = "processContents"; //$NON-NLS-1$

  private static final String DEPRECATED__ATTR = "deprecated"; //$NON-NLS-1$

  private static final String TYPE__ATTR = "type"; //$NON-NLS-1$
  
  private static final String WHEN_CONTAINING_LINE_IS_EMPTY__ATTR = "whenContainingLineIsEmpty"; //$NON-NLS-1$
  
  private static final String REMOVE = "remove"; //$NON-NLS-1$
  
  private final String name;

  private final CustomTagKind kind;

  private final IConfigurationElement configElement;

  private final Map attributeDefinitions;

  private final boolean deprecated;

  private final boolean customContentProcessing;

  private final boolean allowAsEmpty;

  private final TagLibrary tagLibrary;

  private final String description;

  private final boolean removeWhenContainingLineIsEmpty;

  public TagDefinitionImpl(TagLibrary tagLibrary, String tagName, CustomTagKind kind,
    String description, boolean customContentProcessing, boolean allowAsEmpty, 
    boolean deprecated, boolean removeWhenContainingLineIsEmpty )
  {
    this.tagLibrary = tagLibrary;
    this.name = tagName;
    this.kind = kind;
    this.description = description;
    this.customContentProcessing = customContentProcessing;
    this.allowAsEmpty = allowAsEmpty;
    this.deprecated = deprecated;
    this.removeWhenContainingLineIsEmpty = removeWhenContainingLineIsEmpty;
    attributeDefinitions = new HashMap();
    configElement = null;
  }
  /**
   * 
   * @param tagName the name of the tag
   * @param configElement the configuration defining the tag
   * @param kind the kind of the tag
   * @param tagLibrary the parent tag library
   */
  public TagDefinitionImpl(String tagName, IConfigurationElement configElement, CustomTagKind kind, TagLibrary tagLibrary)
  {
    super();
    this.configElement = configElement;
    this.kind = kind;
    name = tagName;
    this.tagLibrary = tagLibrary;

    deprecated = Boolean.valueOf(configElement.getAttribute(DEPRECATED__ATTR)).booleanValue();
    customContentProcessing = CONTENT_PROCESSING__VALUE__CUSTOM.equals(configElement.getAttribute(CONTENT_PROCESSING__ATTR));
    allowAsEmpty = Boolean.valueOf(configElement.getAttribute(ALLOW_AS_EMPTY__ATTR)).booleanValue();

    String removeWhenEmpty = configElement.getAttribute(WHEN_CONTAINING_LINE_IS_EMPTY__ATTR);
    if(removeWhenEmpty == null)
    {
      removeWhenContainingLineIsEmpty = 
        kind != CustomTagKind.EMPTY && kind != CustomTagKind.FUNCTION;
    }
    else 
    {
      removeWhenContainingLineIsEmpty = REMOVE.equals(removeWhenEmpty);
    }
    
    description = ExtensionTagLibraryImpl.getDescription(configElement);

    IConfigurationElement[] children = configElement.getChildren(ATTRIBUTE__ELEMENT);
    attributeDefinitions = new HashMap(children.length);
    for (int i = 0; i < children.length; i++)
    {
      TagAttributeDefinition tad = new TagAttributeDefinitionImpl(
        children[i].getAttribute(NAME__ATTR),
        USE__VALUE__REQUIRED.equalsIgnoreCase(children[i].getAttribute(USE__ATTR)),
        Boolean.valueOf(children[i].getAttribute(DEPRECATED__ATTR)).booleanValue(),
        ExtensionTagLibraryImpl.getDescription(children[i]),
        children[i].getAttribute(TYPE__ATTR));
      if (tad.getName() != null)
      {
        addTagAttribute(tad);
      }
      else
      {
        String msg = JET2Messages.TagDefinitionImpl_MissingName;
        InternalJET2Platform.logError(MessageFormat.format(msg, new Object []{
          children[i].getDeclaringExtension().getExtensionPointUniqueIdentifier(),
          children[i].getDeclaringExtension().getNamespace() }));
      }
    }
  }
  /**
   * @param tagAttributeDefinition
   */
  public void addTagAttribute(TagAttributeDefinition tagAttributeDefinition)
  {
    attributeDefinitions.put(tagAttributeDefinition.getName(), tagAttributeDefinition);
  }

  /* (non-Javadoc)
   * @see org.eclipse.jet.taglib.TagDefinition#getName()
   */
  public final String getName()
  {
    return name;
  }

  /* (non-Javadoc)
   * @see org.eclipse.jet.taglib.TagDefinition#getKind()
   */
  public CustomTagKind getKind()
  {
    return kind;
  }

  public CustomTag newTagElement() throws CoreException
  {
    return configElement != null ? (CustomTag)configElement.createExecutableExtension(CLASS__ATTR) : null;
  }

  public TagAttributeDefinition getAttributeDefinition(String attrName)
  {
    return (TagAttributeDefinition)attributeDefinitions.get(attrName);
  }

  public List getAttributeDefinitions()
  {
    List list = new ArrayList(attributeDefinitions.values());
    Collections.sort(list, new Comparator()
      {

        public int compare(Object o1, Object o2)
        {
          TagAttributeDefinition ad1 = (TagAttributeDefinition)o1;
          TagAttributeDefinition ad2 = (TagAttributeDefinition)o2;
          return ad1.getName().compareTo(ad2.getName());
        }
      });
    return list;
  }

  public boolean isDeprecated()
  {
    return deprecated;
  }

  public boolean requiresNewWriter()
  {
    boolean required = false;
    if (kind == CustomTagKind.FUNCTION)
    {
      required = true;
    }
    else if (customContentProcessing
      && (kind == CustomTagKind.CONTAINER || kind == CustomTagKind.CONDITIONAL || kind == CustomTagKind.ITERATING))
    {
      required = true;
    }
    return required;
  }

  public boolean isEmptyTagAllowed()
  {
    boolean allowed = false;
    if (kind == CustomTagKind.EMPTY)
    {
      allowed = true;
    }
    else if (allowAsEmpty && (kind == CustomTagKind.CONTAINER || kind == CustomTagKind.CONDITIONAL || kind == CustomTagKind.ITERATING))
    {
      allowed = true;
    }
    return allowed;
  }

  public boolean isContentAllowed()
  {
    boolean allowed = false;
    if (kind == CustomTagKind.FUNCTION || kind == CustomTagKind.CONTAINER || kind == CustomTagKind.CONDITIONAL
      || kind == CustomTagKind.ITERATING || kind == CustomTagKind.OTHER)
    {
      allowed = true;
    }
    return allowed;
  }

  public TagLibrary getTagLibrary()
  {
    return tagLibrary;
  }

  public String getDescription()
  {
    return description;
  }
  
  public boolean removeWhenContainingLineIsEmpty()
  {
      return removeWhenContainingLineIsEmpty;
  }
}
