/**
 * <copyright>
 *
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 *   IBM - Initial API and implementation
 *
 * </copyright>
 *
 * $Id$
 */
package org.eclipse.jet.internal.taglib.java;

import java.net.MalformedURLException;
import java.net.URL;
import java.text.MessageFormat;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.emf.codegen.jmerge.JControlModel;
import org.eclipse.emf.codegen.jmerge.JMerger;
import org.eclipse.emf.codegen.jmerge.PropertyMerger;
import org.eclipse.jdt.core.jdom.DOMFactory;
import org.eclipse.jdt.core.jdom.IDOMCompilationUnit;
import org.eclipse.jdt.core.jdom.IDOMFactory;
import org.eclipse.jet.IWriterListener;
import org.eclipse.jet.JET2Context;
import org.eclipse.jet.JET2Writer;
import org.eclipse.jet.internal.l10n.JET2Messages;
import org.eclipse.jet.taglib.AbstractEmptyTag;
import org.eclipse.jet.taglib.JET2TagException;
import org.eclipse.jet.taglib.TagInfo;
import org.eclipse.jet.taglib.TagUtil;
import org.eclipse.jet.transform.TransformContextExtender;
import org.eclipse.jface.text.Position;

/**
 * Implement the Java tag &lt;java:merge&gt;.
 */
public class MergeTag extends AbstractEmptyTag
{

  private final String EMF_CODEGEN_RULES_URL = "platform:/plugin/org.eclipse.emf.codegen.ecore/templates/emf-merge.xml"; //$NON-NLS-1$
  
  /**
   * Position marker for JMergeable content.
   */
  public static class MergePosition extends Position
  {

    private final URL rulesURL;

    public MergePosition(URL rulesURL, int offset)
    {
      super(offset);
      this.rulesURL = rulesURL;
      
    }

    /**
     * @return Returns the rulesURL.
     */
    public final URL getRulesURL()
    {
      return rulesURL;
    }

  }

/**
   * Process a JMergeTag on the template content.
   */
  public static class JMergeProcessor implements IWriterListener
  {

    /**
     * 
     */
    public JMergeProcessor()
    {
      super();
    }

    /* (non-Javadoc)
     * @see org.eclipse.jet.IWriterListener#finalizeContent(org.eclipse.jet.JET2Writer, java.lang.Object)
     */
    public void finalizeContent(JET2Writer writer, Object file) throws JET2TagException
    {
      if(file instanceof IFile)
      {
        IFile iFile = (IFile)file;
        if(!iFile.exists())
        {
          return;
        }
        if("java".equals(iFile.getFileExtension())) //$NON-NLS-1$
        {
          jmerge(writer, iFile);
        }
        else if("properties".equals(iFile.getFileExtension())) //$NON-NLS-1$
        {
          propmerge(writer, iFile);
        }
      }
    }

    private void propmerge(JET2Writer writer, IFile file) throws JET2TagException
    {
      PropertyMerger merger = new PropertyMerger();
      
      try
      {
        String targetPropertiesString = TagUtil.getContents(file.getLocation());
        
        merger.setSourceProperties(writer.getDocument().get());
        merger.setTargetProperties(targetPropertiesString);
        
        merger.merge();
        
        writer.getDocument().set(merger.getTargetProperties());
      }
      catch (CoreException e)
      {
        final String msg = JET2Messages.MergeTag_CouldNotRead;
        throw new JET2TagException(MessageFormat.format(msg, new Object []{ file.getFullPath() }));
      }
    }

    /**
     * @param writer
     * @param iFile
     * @throws JET2TagException
     */
    private void jmerge(JET2Writer writer, IFile iFile) throws JET2TagException
    {
      final Position[] positions = writer.getPositions(JMERGE_POSITION_CATEGORY);
      
      try
      {
        String targetCUSource = TagUtil.getContents(iFile.getLocation());
        IDOMFactory domFactory = new DOMFactory();

        IDOMCompilationUnit sourceCU = domFactory.createCompilationUnit(writer.toString(), iFile.getName());

        IDOMCompilationUnit targetCU = domFactory.createCompilationUnit(targetCUSource, iFile.getName());
        JControlModel controlModel = new JControlModel(((MergePosition)positions[0]).getRulesURL().toExternalForm());
        JMerger merger = new JMerger(controlModel, sourceCU, targetCU);

        merger.merge();

        writer.getDocument().set(targetCU.getContents());
      }
      catch (CoreException e)
      {
        final String msg = JET2Messages.MergeTag_CouldNotRead;
        throw new JET2TagException(MessageFormat.format(msg, new Object []{ iFile.getFullPath() }));
      }
    }

    /* (non-Javadoc)
     * @see org.eclipse.jet.IWriterListener#postCommitContent(org.eclipse.jet.JET2Writer, java.lang.Object)
     */
    public void postCommitContent(JET2Writer writer, Object file)
    {
      // nothing to do.
    }

  }

private static final String JMERGE_POSITION_CATEGORY = MergeTag.class.getName();;

  /**
   * 
   */
  public MergeTag()
  {
    super();
  }

  /* (non-Javadoc)
   * @see org.eclipse.jet.taglib.EmptyTag#doAction(org.eclipse.jet.taglib.TagInfo, org.eclipse.jet.JET2Context, org.eclipse.jet.JET2Writer)
   */
  public void doAction(TagInfo td, JET2Context context, JET2Writer out) throws JET2TagException
  {
    String rules = getAttribute("rules"); //$NON-NLS-1$
    if(rules == null)
    {
      rules = EMF_CODEGEN_RULES_URL;
    }
    String rulesContext = getAttribute("rulesContext"); //$NON-NLS-1$
    if(rulesContext == null)
    {
      rulesContext = "transform"; //$NON-NLS-1$
    }
    
    final URL baseURL = TransformContextExtender.getInstance(context).getBaseURL(rulesContext);
    
    if(rules.startsWith("/")) //$NON-NLS-1$
    {
      rules = rules.substring(1);
    }
    
    try
    {
      URL rulesURL = new URL(baseURL, rules);
      
      out.getDocument().addPositionCategory(JMERGE_POSITION_CATEGORY);
      
      out.addPosition(JMERGE_POSITION_CATEGORY, new MergePosition(rulesURL, out.getDocument().getLength()));
      
      out.addEventListener(JMERGE_POSITION_CATEGORY, new JMergeProcessor());
    }
    catch (MalformedURLException e)
    {
      String msg = JET2Messages.MergeTag_CouldNotCreateURL;
      throw new JET2TagException(MessageFormat.format(msg, 
        new Object[] {baseURL.toExternalForm(), rules}));
    }
  }

}
